@@ -18,20 +18,20 @@ So, in summary: Col objects are what you define, but SOCol objects
 are what gets used.
 """
 
-import re, time
 from array import array
+from itertools import count
+import re, time
 try:
     import cPickle as pickle
 except ImportError:
     import pickle
-import sqlbuilder
 # Sadly the name "constraints" conflicts with many of the function
 # arguments in this module, so we rename it:
-import constraints as consts
-from formencode import compound
-from formencode import validators
+from formencode import compound, validators
 from classregistry import findClass
-from itertools import count
+import constraints as constrs
+import sqlbuilder
+from styles import capword
 
 NoDefault = sqlbuilder.NoDefault
 
@@ -149,7 +149,7 @@ class SOCol(object):
         elif notNone is not NoDefault:
             self.notNone = notNone
         if self.notNone:
-            self.constraints = [consts.notNull] + self.constraints
+            self.constraints = [constrs.notNull] + self.constraints
 
         self.name = name
         self.soClass = soClass
@@ -189,7 +189,7 @@ class SOCol(object):
         else:
             self.unique = unique
         if self.unique and alternateMethodName is None:
-            self.alternateMethodName = 'by' + self.name[0].capitalize() + self.name[1:]
+            self.alternateMethodName = 'by' + capword(self.name)
         else:
             self.alternateMethodName = alternateMethodName
 
@@ -427,9 +427,9 @@ class SOStringLikeCol(SOCol):
         super(SOStringLikeCol, self).__init__(**kw)
 
     def autoConstraints(self):
-        constraints = [consts.isString]
+        constraints = [constrs.isString]
         if self.length is not None:
-            constraints += [consts.MaxLength(self.length)]
+            constraints += [constrs.MaxLength(self.length)]
         return constraints
 
     def _sqlType(self):
@@ -623,7 +623,7 @@ class SOIntCol(SOCol):
         SOCol.__init__(self, **kw)
 
     def autoConstraints(self):
-        return [consts.isInt]
+        return [constrs.isInt]
 
     def createValidators(self):
         return [IntValidator(name=self.name)] + \
@@ -693,7 +693,7 @@ class BoolValidator(validators.Validator):
 
 class SOBoolCol(SOCol):
     def autoConstraints(self):
-        return [consts.isBool]
+        return [constrs.isBool]
 
     def createValidators(self):
         return [BoolValidator(name=self.name)] + \
@@ -746,7 +746,7 @@ class SOFloatCol(SOCol):
     # 3-03 @@: support precision (e.g., DECIMAL)
 
     def autoConstraints(self):
-        return [consts.isFloat]
+        return [constrs.isFloat]
 
     def createValidators(self):
         return [FloatValidator(name=self.name)] + \
@@ -768,6 +768,10 @@ class SOKeyCol(SOCol):
     # 3-03 @@: this should have a simplified constructor
     # Should provide foreign key information for other DBs.
 
+    def __init__(self, **kw):
+        self.refColumn = kw.pop('refColumn', None)
+        super(SOKeyCol, self).__init__(**kw)
+
     def _sqlType(self):
         return self.key_type[self.soClass.sqlmeta.idType]
 
@@ -788,18 +792,18 @@ class SOForeignKey(SOKeyCol):
     def __init__(self, **kw):
         foreignKey = kw['foreignKey']
         style = kw['soClass'].sqlmeta.style
-        if not kw.get('name'):
-            kw['name'] = style.instanceAttrToIDAttr(style.pythonClassToAttr(foreignKey))
-        else:
+        if kw.get('name'):
             kw['origName'] = kw['name']
             kw['name'] = style.instanceAttrToIDAttr(kw['name'])
+        else:
+            kw['name'] = style.instanceAttrToIDAttr(style.pythonClassToAttr(foreignKey))
         super(SOForeignKey, self).__init__(**kw)
 
     def sqliteCreateSQL(self):
         sql = SOKeyCol.sqliteCreateSQL(self)
         other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
         tName = other.sqlmeta.table
-        idName = other.sqlmeta.idName
+        idName = self.refColumn or other.sqlmeta.idName
         if self.cascade is not None:
             if self.cascade == 'null':
                 action = 'ON DELETE SET NULL'
@@ -828,7 +832,7 @@ class SOForeignKey(SOKeyCol):
         sTName = self.soClass.sqlmeta.table
         other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
         tName = other.sqlmeta.table
-        idName = other.sqlmeta.idName
+        idName = self.refColumn or other.sqlmeta.idName
         if self.cascade is not None:
             if self.cascade == 'null':
                 action = 'ON DELETE SET NULL'
@@ -854,7 +858,7 @@ class SOForeignKey(SOKeyCol):
         sTLocalName = sTName.split('.')[-1]
         other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
         tName = other.sqlmeta.table
-        idName = other.sqlmeta.idName
+        idName = self.refColumn or other.sqlmeta.idName
         if self.cascade is not None:
             if self.cascade == 'null':
                 action = 'ON DELETE SET NULL'
@@ -883,7 +887,7 @@ class SOForeignKey(SOKeyCol):
         sql = SOKeyCol.sybaseCreateSQL(self)
         other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
         tName = other.sqlmeta.table
-        idName = other.sqlmeta.idName
+        idName = self.refColumn or other.sqlmeta.idName
         reference = ('REFERENCES %(tName)s(%(idName)s) ' %
                      {'tName':tName,
                       'idName':idName})
@@ -898,7 +902,7 @@ class SOForeignKey(SOKeyCol):
         sql = SOKeyCol.mssqlCreateSQL(self, connection)
         other = findClass(self.foreignKey, self.soClass.sqlmeta.registry)
         tName = other.sqlmeta.table
-        idName = other.sqlmeta.idName
+        idName = self.refColumn or other.sqlmeta.idName
         reference = ('REFERENCES %(tName)s(%(idName)s) ' %
                      {'tName':tName,
                       'idName':idName})
@@ -915,7 +919,7 @@ class SOForeignKey(SOKeyCol):
         #I assume that foreign key name is identical to the id of the reference table
         sql = ' '.join([fidName, self._maxdbType()])
         tName = other.sqlmeta.table
-        idName  = other.sqlmeta.idName
+        idName  = self.refColumn or other.sqlmeta.idName
         sql=sql + ',' + '\n'
         sql=sql + 'FOREIGN KEY (%s) REFERENCES %s(%s)'%(fidName,tName,idName)
         return sql
@@ -953,7 +957,7 @@ class SOEnumCol(SOCol):
         super(SOEnumCol, self).__init__(**kw)
 
     def autoConstraints(self):
-        return [consts.isString, consts.InList(self.enumValues)]
+        return [constrs.isString, constrs.InList(self.enumValues)]
 
     def createValidators(self):
         return [EnumValidator(name=self.name, enumValues=self.enumValues,
@@ -1033,7 +1037,7 @@ class SOSetCol(SOCol):
         super(SOSetCol, self).__init__(**kw)
 
     def autoConstraints(self):
-        return [consts.isString, consts.InList(self.setValues)]
+        return [constrs.isString, constrs.InList(self.setValues)]
 
     def createValidators(self):
         return [SetValidator(name=self.name, setValues=self.setValues)] + \
