@@ -59,10 +59,6 @@ class SQLiteConnection(DBAPI):
         if self.using_sqlite2:
             if autoCommit:
                 opts["isolation_level"] = None
-            opts["detect_types"] = sqlite.PARSE_DECLTYPES
-            for col_type in "text", "char", "varchar", "date", "time", "datetime", "timestamp":
-                sqlite.register_converter(col_type, stop_pysqlite2_converting_strings)
-                sqlite.register_converter(col_type.upper(), stop_pysqlite2_converting_strings)
             global sqlite2_Binary
             if sqlite2_Binary is None:
                 sqlite2_Binary = sqlite.Binary
@@ -95,7 +91,11 @@ class SQLiteConnection(DBAPI):
         if self._memory:
             self._memoryConn = sqlite.connect(
                 self.filename, **self._connOptions)
+            # Convert text data from SQLite to str, not unicode -
+            # SQLObject converts it to unicode itself.
+            self._memoryConn.text_factory = str
 
+    @classmethod
     def _connectionFromParams(cls, user, password, host, port, path, args):
         assert host is None and port is None, (
             "SQLite can only be used locally (with a URI like "
@@ -107,7 +107,6 @@ class SQLiteConnection(DBAPI):
         if path == "/:memory:":
             path = ":memory:"
         return cls(filename=path, **args)
-    _connectionFromParams = classmethod(_connectionFromParams)
 
     def oldUri(self):
         path = self.filename
@@ -140,7 +139,7 @@ class SQLiteConnection(DBAPI):
             return conn
         threadid = thread.get_ident()
         if (self._pool is not None
-            and self._threadPool.has_key(threadid)):
+            and threadid in self._threadPool):
             conn = self._threadPool[threadid]
             del self._threadPool[threadid]
             if conn in self._pool:
@@ -164,7 +163,7 @@ class SQLiteConnection(DBAPI):
         threadid = self._threadOrigination.get(id(conn))
         DBAPI.releaseConnection(self, conn, explicit=explicit)
         if (self._pool is not None and threadid
-            and not self._threadPool.has_key(threadid)):
+            and threadid not in self._threadPool):
             self._threadPool[threadid] = conn
         else:
             if self._pool and conn in self._pool:
@@ -188,7 +187,9 @@ class SQLiteConnection(DBAPI):
     def makeConnection(self):
         if self._memory:
             return self._memoryConn
-        return self.module.connect(self.filename, **self._connOptions)
+        conn = self.module.connect(self.filename, **self._connOptions)
+        conn.text_factory = str # Convert text data to str, not unicode
+        return conn
 
     def _executeRetry(self, conn, cursor, query):
         if self.debug:
@@ -247,13 +248,13 @@ class SQLiteConnection(DBAPI):
         else:
             return DBAPI._insertSQL(self, table, names, values)
 
+    @classmethod
     def _queryAddLimitOffset(cls, query, start, end):
         if not start:
             return "%s LIMIT %i" % (query, end)
         if not end:
             return "%s LIMIT 0 OFFSET %i" % (query, start)
         return "%s LIMIT %i OFFSET %i" % (query, end-start, start)
-    _queryAddLimitOffset = classmethod(_queryAddLimitOffset)
 
     def createColumn(self, soClass, col):
         return col.sqliteCreateSQL()
@@ -394,7 +395,3 @@ class SQLiteConnection(DBAPI):
         if self._memory:
             return
         os.unlink(self.filename)
-
-
-def stop_pysqlite2_converting_strings(s):
-    return s
